package com.jjt.elec.service.impl;

import com.jjt.elec.domain.ElecPeriod;
import com.jjt.elec.domain.ElecPrice;
import com.jjt.elec.mapper.ElecPriceMapper;
import com.jjt.elec.service.IElecPeriodService;
import com.jjt.elec.service.IElecPriceService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 电费价格配置Service业务层处理
 *
 * @author wukai
 * @date 2025-03-14
 */
@Service
public class ElecPriceServiceImpl implements IElecPriceService {
    @Resource
    private ElecPriceMapper elecPriceMapper;
    @Resource
    private IElecPeriodService periodService;

    /**
     * 查询电费价格配置
     *
     * @param priceId 电费价格配置主键
     * @return 电费价格配置
     */
    @Override
    public ElecPrice selectElecPriceByPriceId(Long priceId) {
        return elecPriceMapper.selectElecPriceByPriceId(priceId);
    }

    /**
     * 查询电费价格配置列表
     *
     * @param elecPrice 电费价格配置
     * @return 电费价格配置
     */
    @Override
    public List<ElecPrice> selectElecPriceList(ElecPrice elecPrice) {
        return elecPriceMapper.selectElecPriceList(elecPrice);
    }

    /**
     * 新增电费价格配置
     *
     * @param elecPrice 电费价格配置
     * @return 结果
     */
    @Override
    public int insertElecPrice(ElecPrice elecPrice) {
        return elecPriceMapper.insertElecPrice(elecPrice);
    }

    /**
     * 修改电费价格配置
     *
     * @param elecPrice 电费价格配置
     * @return 结果
     */
    @Override
    public int updateElecPrice(ElecPrice elecPrice) {
        return elecPriceMapper.updateElecPrice(elecPrice);
    }

    /**
     * 批量删除电费价格配置
     *
     * @param priceIds 需要删除的电费价格配置主键
     * @return 结果
     */
    @Override
    public int deleteElecPriceByPriceIds(Long[] priceIds) {
        return elecPriceMapper.deleteElecPriceByPriceIds(priceIds);
    }

    /**
     * 删除电费价格配置信息
     *
     * @param priceId 电费价格配置主键
     * @return 结果
     */
    @Override
    public int deleteElecPriceByPriceId(Long priceId) {
        return elecPriceMapper.deleteElecPriceByPriceId(priceId);
    }

    /**
     * 根据月份和时段获取价格
     *
     * @param localDate 时间，只取月份
     * @param hour      小时
     * @return 电价
     */
    @Override
    public BigDecimal getPrice(LocalDate localDate, int hour) {
        ElecPeriod period = periodService.selectElecPeriodByPeriodId(1L);
        int year = localDate.getYear();
        int month = localDate.getMonthValue();
        ElecPrice priceObj = selectElecPriceByMonth(year, month);

        // 判断月份是春秋还是夏冬
        boolean isSummerAutumn = getNumbers(period.getSaMonths(), true).contains(month);

        // 根据月份和时段获取价格
        if (isSummerAutumn) {
            return getPriceByPeriod(period.getSaPeak(), period.getSaFlat(), period.getSaValley(), period.getSaSuperPeak(), hour, priceObj);
        } else {
            return getPriceByPeriod(period.getSwPeak(), period.getSwFlat(), period.getSwValley(), period.getSwSuperPeak(), hour, priceObj);
        }
    }

    /**
     * 根据时段获取价格
     */
    private BigDecimal getPriceByPeriod(String peakPeriod, String flatPeriod, String valleyPeriod, String superPeakPeriod, int hour, ElecPrice priceObj) {
        if (getNumbers(peakPeriod, false).contains(hour)) {
            return priceObj.getPricePeak();
        } else if (getNumbers(flatPeriod, false).contains(hour)) {
            return priceObj.getPriceFlat();
        } else if (getNumbers(valleyPeriod, false).contains(hour)) {
            return priceObj.getPriceValley();
        } else if (getNumbers(superPeakPeriod, false).contains(hour)) {
            return priceObj.getPriceSuperPeak();
        }
        return BigDecimal.ZERO;
    }

    /**
     * 解析 1-8,9-11这种数据
     * 如果是月份,则包含头尾，如果是小时，则不包含尾部
     *
     * @param d    字符串
     * @param flag 是否月份
     * @return
     */
    private Set<Long> getNumbers(String d, boolean flag) {
        String[] temp = d.split(",");
        Set<Long> list = new HashSet();
        for (String s : temp) {
            String[] ss = s.split("-");
            if (ss.length > 1) {
                long st = Long.parseLong(ss[0]);
                long ed = Long.parseLong(ss[1]);

                if (!flag) {
                    ed = ed - 1;
                }
                for (long i = st; i <= ed; i++) {
                    list.add(i);
                }
            } else {
                list.add(Long.parseLong(s));
            }
        }
        return list;
    }

    private ElecPrice selectElecPriceByMonth(int year, int month) {
        String monthStr = year + "-" + (month < 10 ? "0" + month : month);
        ElecPrice search = new ElecPrice();
        search.setPriceMonth(monthStr);
        List<ElecPrice> list = selectElecPriceList(search);
        if (list.size() != 0) {
            return list.get(0);
        }
        return null;
    }
}
